import { LogLevel } from '@genesislcap/foundation-utils';

export declare interface ConfirmingAction {
    readonly label: String;
    readonly action: Function;
}

export declare interface DismissingAction {
    readonly label: String;
    readonly action?: Function;
}

export declare interface ErrorBanner {
    readonly dismissingAction?: DismissingAction;
    readonly confirmingActions?: Array<ConfirmingAction>;
}

export declare class ErrorBannerBuilder {
    private dismissingAction;
    private confirmingActions;
    withDismissingAction(dismissingAction: DismissingAction): ErrorBannerBuilder;
    withConfirmingAction(confirmingAction: ConfirmingAction): ErrorBannerBuilder;
    build(): ErrorBanner;
}

export declare enum ErrorBoundaryEvent {
    ERROR_BOUNDARY_EVENT = "error-boundary-event"
}

export declare class ErrorBuilder {
    private title;
    private errorDetails;
    private logLevel;
    private alert;
    private notification;
    private banner;
    private snackbar;
    private dialog;
    withTitle(title: string): ErrorBuilder;
    withErrorDetails(errorDetails: string): ErrorBuilder;
    withLogLevel(logLevel: LogLevel): ErrorBuilder;
    withAlert(alert: boolean): ErrorBuilder;
    withNotification(notification: ErrorNotification): ErrorBuilder;
    withBanner(banner: ErrorBanner): ErrorBuilder;
    withSnackbar(snackbar: Snackbar): ErrorBuilder;
    withDialog(dialog: ErrorDialog): ErrorBuilder;
    build(): ErrorStructure;
}

export declare interface ErrorDialog {
    readonly confirmingActions?: Array<ConfirmingAction>;
    readonly dismissingAction?: DismissingAction;
}

export declare class ErrorDialogBuilder {
    private dismissingAction;
    private confirmingActions;
    withDismissingAction(dismissingAction: DismissingAction): ErrorDialogBuilder;
    withConfirmingAction(confirmingAction: ConfirmingAction): ErrorDialogBuilder;
    build(): ErrorDialog;
}

export declare type ErrorDialogElement = ErrorElement & {
    show: Function;
};

export declare type ErrorElement = HTMLElement & {
    error: ErrorStructure;
};

export declare interface ErrorNotification {
    readonly type?: 'warning' | 'success' | '';
    readonly autoClose?: boolean;
    readonly closeTimeout?: number;
}

export declare interface ErrorStructure {
    readonly title: string;
    readonly errorDetails: string;
    readonly logLevel?: LogLevel;
    readonly config?: {
        /**
         * @deprecated Use logLevel from the top level instead
         */
        readonly logLevel?: LogLevel;
        readonly alert?: boolean;
        readonly notification?: ErrorNotification;
        readonly banner?: ErrorBanner;
        readonly snackbar?: Snackbar;
        readonly dialog?: ErrorDialog;
    };
}

export declare const getErrorBannerBuilder: () => ErrorBannerBuilder;

export declare const getErrorBuilder: () => ErrorBuilder;

/**
 * Get/Create the error container in the DOM
 * @public
 * @param id - The id of the container
 * @param tagName - The tag name to create prefix for the design system provider
 * @returns
 */
export declare const getErrorContainer: (id: string, tagName: String) => HTMLElement;

/**
 * Retrieves the error details from an ErrorStructure object
 * @public
 * @param details - The ErrorStructure object
 * @returns The title and error details concatenated as a string, or default values ('Error: Something went wrong') if not present
 */
export declare const getErrorDetails: (details: ErrorStructure) => string;

export declare const getErrorDialogBuilder: () => ErrorDialogBuilder;

/**
 * Retrieves the log level from an ErrorStructure object
 * @public
 * @param details - The ErrorStructure object
 * @returns The log level or the default value (LogLevel.Error) if not present
 */
export declare const getLoglevel: (details: ErrorStructure) => LogLevel;

export declare const getNotificationBuilder: () => NotificationBuilder;

export declare const getSnackbarBuilder: () => SnackbarBuilder;

export declare class NotificationBuilder {
    private autoClose;
    private closeTimeout;
    private type;
    withAutoClose(autoClose: boolean): NotificationBuilder;
    withCloseTimeout(closeTimeout: number): NotificationBuilder;
    withType(type: 'warning' | 'success' | ''): NotificationBuilder;
    build(): ErrorNotification;
}

/**
 * Shows the error dialog
 * @public
 * @param error - The ErrorStructure object
 * @param tagName - The tag name to create prefix for the design system provider & error component
 * @returns
 * */
export declare const showDialog: (error: ErrorStructure, tagName: String) => void;

/**
 * Shows the error in the appropriate component
 * @public
 * @param error - The ErrorStructure object
 * @param tagName - The tag name to create prefix for the design system provider *
 * @returns
 * */
export declare const showError: (error: ErrorStructure, tagName: String) => void;

/**
 * Shows the error notification
 * @public
 * @param error - The ErrorStructure object
 * @param tagName - The tag name to create prefix for the design system provider & error component
 * @returns
 * */
export declare const showNotification: (error: ErrorStructure, tagName: String) => void;

/**
 * Shows the snackbar
 * @public
 * @param error - The ErrorStructure object
 * @param tagName - The tag name to create prefix for the design system provider & error component
 * @returns
 * */
export declare const showSnackbar: (error: ErrorStructure, tagName: String) => void;

export declare interface Snackbar {
    readonly type?: 'error' | '';
    readonly confirmingActions?: Array<ConfirmingAction>;
}

export declare class SnackbarBuilder {
    private confirmingActions;
    private type;
    /**
     * Add type to snackbar
     * @public
     * @param type
     * @returns
     */
    withType(type: 'error' | ''): SnackbarBuilder;
    /**
     * Add confirming action to snackbar
     * @public
     * @param confirmingAction
     * @returns
     * */
    withConfirmingAction(confirmingAction: ConfirmingAction): SnackbarBuilder;
    /**
     * Build snackbar
     * @public
     * @returns Snackbar
     * */
    build(): Snackbar;
}

export { }
